/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package langid;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 *
 * @author cle-32-pc
 */
public class LangID {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws IOException {
        /**
         * Tests the Language ID package on the validation datasest. Reports the
         * Mean Absolute Error (MAE) over all examples, Arabic ones alone, and
         * English ones alone. Reports the size of the language model and
         * stopword list, and the time taken to load each, respectively. Reports
         * the total time taken in language proportion estimation, and the time
         * taken per page.
         */
        
        String content = "پاکستان میرا ملک ہے";

        LangID langid = new LangID();

        HashMap<String, Integer> lm = langid.loadLM();		// loads language model from disk
        HashMap<String, Integer> sw = langid.loadSW();          // loads stopwords list from disk

        System.out.println(langid.identifyLang(content, lm, sw).get("ur"));

        
//        File folder = new File("./use-files/");
//        File[] listOfFiles = folder.listFiles();
//        HashMap<String, String> webpages = new HashMap<>();
//
//        final long LMstartTime = System.currentTimeMillis();
//        HashMap<String, Integer> lm = LangID.loadLM();
//        final long LMendTime = System.currentTimeMillis();
//
//        final long SWstartTime = System.currentTimeMillis();
//        HashMap<String, Integer> sw = LangID.loadSW();
//        final long SWendTime = System.currentTimeMillis();
//
//        final long startTime = System.currentTimeMillis();
//        for (File file : listOfFiles) {
//            if (file.isFile()) {
//                String line;
//                String content = "";
//                BufferedReader reader = new BufferedReader(new FileReader(file));
//                while ((line = reader.readLine()) != null) {
//                    content = content + line;
//                }
//                webpages.put(file.getName(), new LangID().identifyLang(content, lm, sw).get("ur") + "");
//            }
//        }
//        final long endTime = System.currentTimeMillis();
//
//        List<Float> mae_all = new ArrayList<Float>();
//        List<Float> mae_en = new ArrayList<Float>();
//        List<Float> mae_ar = new ArrayList<Float>();
//
//        for (String key : webpages.keySet()) {
////            float truth = Float.parseFloat(key.split("_")[1].substring(1));
//            float pred = Float.parseFloat(webpages.get(key));
////            float diff = Math.abs(pred - truth);
////            mae_all.add(diff);
//            System.out.println(key + "\t" + webpages.get(key));
////            System.out.println(key + "\t" + key.split("_")[1].substring(1) + "\t" + webpages.get(key));
////
////            if (key.split("_")[0].equals("en")) {// && (key.split("_")[2].substring(1).equals("500"))) {
////                mae_en.add(diff);
////            }
////            if (key.split("_")[0].equals("ar")) {// && (key.split("_")[2].substring(1).equals("500"))) {
////                mae_ar.add(diff);
////            }
//        }
//
//        System.out.println("\nLoading LM (ms):\t" + (LMendTime - LMstartTime));
//        System.out.println("Loading SW (ms):\t" + (SWendTime - SWstartTime));
//
//        System.out.println("\nLanguage Model size:\t" + lm.size());
//        System.out.println("Stopwords size:\t\t" + sw.size());
//
////        System.out.println("\nMAE:\t\t" + new LangID().calculateAverage(mae_all) + "\t" + mae_all.size());
////        System.out.println("EN:\t\t" + new LangID().calculateAverage(mae_en) + "\t" + mae_en.size());
////        System.out.println("AR:\t\t" + new LangID().calculateAverage(mae_ar) + "\t" + mae_ar.size());
//        System.out.println("\nEstimating Language (ms):\t" + (endTime - startTime));
//        System.out.println("Total pages:\t\t\t" + webpages.size());
//        System.out.println("Time per page (ms):\t\t" + (endTime - startTime) / (float) webpages.size());
         
    }

    public static HashMap<String, Integer> loadLM() throws IOException {
        /**
         * Loads the language model from a UTF-8 text file into a HashMap. Text
         * file must contain each word and its count on its own line, with the
         * two separated by a colon.
         */

//        String filePath = "languagemodel.txt";
//        BufferedReader reader = new BufferedReader(new FileReader(filePath));
        InputStream stream = LangID.class.getResourceAsStream("/languagemodel.txt");
        BufferedReader reader = new BufferedReader(new InputStreamReader(stream));

        HashMap<String, Integer> map = new HashMap<>();
        String line;
        while ((line = reader.readLine()) != null) {
            String[] parts = line.split(":");
//            if (parts.length >= 2) {
            String key = parts[0];
            Integer value = Integer.parseInt(parts[1]);
            map.put(key, value);
//            }
        }
        reader.close();
        return map;
    }

    public static HashMap<String, Integer> loadSW() throws IOException {
        /**
         * Loads the list of stopwords from a UTF-8 text file into a HashMap.
         * Text file must contain all word on one line, separated only by a
         * colon.
         */

//        BufferedReader reader = new BufferedReader(new FileReader(filePath));
//        String filePath = "stopwords.txt";
        InputStream stream = LangID.class.getResourceAsStream("/stopwords.txt");
        BufferedReader reader = new BufferedReader(new InputStreamReader(stream));

        HashMap<String, Integer> map = new HashMap<>();
        String line;

        while ((line = reader.readLine()) != null) {
            String[] parts = line.split(":");
            for (String str : parts) {
                map.put(str, 1);
            }
        }
        reader.close();
        return map;
    }

    public static HashMap<String, Float> identifyLang(String content, HashMap<String, Integer> lm, HashMap<String, Integer> sw) throws IOException {
        /**
         * Checks an input string for the presence of Arabic Unicode,
         * preprocesses it, tokenizes it, checks for the existence of Urdu,
         * estimates the proportion of Urdu.
         */
        HashMap<String, Float> result = new HashMap<>();
        Preprocessor preproc = new Preprocessor();
        ProportionEstimator prop = new ProportionEstimator();

        if (content == null) {
            result.put("ur", (float) 0);
            return result;
        }

        if (!preproc.hasArabicUnicode(content)) {
            result.put("ur", (float) 0);
            return result;
        }

        String normalized = preproc.normalize(content);
        if (normalized == null) {
            result.put("ur", (float) 0);
            return result;
        }

        String[] tokens = preproc.tokenize(normalized);
        
        if (tokens.length < 2) {
            result.put("ur", prop.estimate_unigram(tokens, lm));
            return result;
        }

        if (!prop.hasUrdu(tokens, sw)) {
            result.put("ur", (float) 0);
            return result;
        }
        
        List Urdufound = prop.predUrdu(tokens, lm);

        List smoothed = prop.smooth(Urdufound, 10);
        
        result.put("ur", prop.getProp(smoothed));
//        result.put("ur", prop.estimate_context(tokens, lm));
        return result;
    }

//    public static float calculateAverage(List<Float> marks) {
//        /**
//         * Calculates the average of a list of floating point numbers.
//         */
//        float sum = 0;
//        if (!marks.isEmpty()) {
//            for (float mark : marks) {
//                sum += mark;
//            }
//            return sum / (float) marks.size();
//        }
//        return sum;
//    }
}
